@@ -1,5 +1,3 @@
-
-
 #' Fit Cox model with partly timevarying effects.
 #' 
 #' Fits proportional hazards model with some effects time-varying and some
@@ -13,7 +11,7 @@
 #' 
 #' The data for a subject is presented as multiple rows or 'observations', each
 #' of which applies to an interval of observation (start, stop].  When counting
-#' process data with the )start,stop] notation is used the 'id' variable is
+#' process data with the )start,stop] notation is used, the 'id' variable is
 #' needed to identify the records for each subject. The program assumes that
 #' there are no ties, and if such are present random noise is added to break
 #' the ties.
@@ -24,6 +22,9 @@
 #' regressors are specified by the wrapper const(), and cluster variables (for
 #' computing robust variances) by the wrapper cluster().
 #' @param data a data.frame with the variables.
+#' @param weights to add weights 
+#' @param subset to subset the data
+#' @param na.action to take care of na' s
 #' @param start.time start of observation period where estimates are computed.
 #' @param max.time end of observation period where estimates are computed.
 #' Estimates thus computed from [start.time, max.time]. Default is max of data.
@@ -100,11 +101,11 @@
 #' plot(out)
 #' 
 ##' @export
-timecox<-function(formula=formula(data),data=sys.parent(),
-start.time=0,max.time=NULL,id=NULL,clusters=NULL,
-n.sim=1000,residuals=0,robust=1,Nit=20,bandwidth=0.5,
-method="basic",weighted.test=0,degree=1,covariance=0)
-{
+timecox<-function(formula=formula(data),data, weights, subset, na.action,
+                  start.time=0,max.time=NULL,id=NULL,clusters=NULL,
+                  n.sim=1000,residuals=0,robust=1,Nit=20,bandwidth=0.5,
+                  method="basic",weighted.test=0,degree=1,covariance=0)
+  {
   sim2<-0; if (n.sim==0) sim<-0 else sim<-1;
   if (method!="basic") stop("Only runs the default method at the moment\n"); 
 
@@ -126,22 +127,28 @@ method="basic",weighted.test=0,degree=1,covariance=0)
   if (n.sim>0 & n.sim<50) {n.sim<-50 ; cat("Minimum 50 simulations\n");}
 
   call <- match.call()
-  m <- match.call(expand.dots=FALSE)
-  m$robust<-m$start.time<-m$degree<-m$weighted.test<-
-    m$method<-m$Nit<-m$bandwidth<-m$max.time<-m$pers<-
-      m$residuals<-m$n.sim<-m$id<-m$covariance<-NULL
 
+  # New code
+  indx <- match(c("formula", "data", "weights", "subset", "na.action",
+                   "id"), names(call), nomatch=0)
+  if (indx[1] ==0) stop ("a formula argument is required")
+  temp <- call[c(1, indx)]  # only keep the arguments we want
+  temp[[1L]] <- quote(stats::model.frame)  # change the function called
   special <- c("const","cluster")
-  Terms <- if(missing(data)) terms(formula, special)
+  temp$formula <- if(missing(data)) terms(formula, special)
   else              terms(formula, special, data=data)
-  m$formula <- Terms
-  m[[1]] <- as.name("model.frame")
-  m <- eval(m, sys.parent())
+
+  m <- eval(temp, parent.frame())
   mt <- attr(m, "terms")
   intercept<-attr(mt, "intercept")
-  Y <- model.extract(m, "response")
+  Y <- model.response(m,)
   if (!inherits(Y, "Surv")) 
     stop("Response must be a survival object")
+  id <- model.extract(m, "(id)")
+  weights <- model.weights(m)
+  if (!is.null(weights)) stop("timecox does not support case weights")
+  Terms <- terms(m)
+  # end new code
 
   des<-read.design(m,Terms)
   X<-des$X; Z<-des$Z; npar<-des$npar; px<-des$px; pz<-des$pz;
