@@ -3,13 +3,20 @@
 #' Simulates data from piecwise constant baseline hazard that can also be of
 #' Cox type. Censor data at highest value of the break points.
 #' 
-#' @param cumhazard cumulative hazard, or piece-constant rates for periods
-#' defined by first column of input.
-#' @param rr number of simulations or vector of relative risk for simuations.
-#' @param n number of simulations given as "n" 
+#' For a piecewise linear cumulative hazard the inverse is easy to compute with 
+#' and delayed entry x we compute 
+#' \deqn{\Lambda^{-1}(\Lambda(x) + E/RR)}, 
+#' where RR are the relative risks and E is exponential with mean 1.
+#' This quantity has survival function 
+#' \deqn{P(T > t | T>x) = exp(-RR (\Lambda(t) - \Lambda(x)))}. 
+#' 
+#' @param cumhazard cumulative hazard, or piece-constant rates for periods defined by first column of input.
+#' @param rr relative risk for simulations, alternatively when rr=1 specify n 
+#' @param n number of simulation if rr not given 
 #' @param entry delayed entry time for simuations.
 #' @param cum.hazard specifies wheter input is cumulative hazard or rates.
 #' @param cause name of cause 
+#' @param extend  to extend piecewise constant with constant rate. Default is average rate over time from cumulative (when TRUE), if numeric then uses given rate.
 #' @author Thomas Scheike
 #' @keywords survival
 #' @examples
@@ -24,14 +31,14 @@
 #' cumhaz <- cumsum(c(0,diff(breaks)*rates[-1]))
 #' cumhaz <- cbind(breaks,cumhaz)
 #' 
-#' pctime <- pc.hazard(haz,n=1000,cum.hazard=FALSE)
+#' pctime <- rchaz(haz,n=1000,cum.hazard=FALSE)
 #' 
 #' par(mfrow=c(1,2))
 #' ss <- aalen(Surv(time,status)~+1,data=pctime,robust=0)
 #' plot(ss)
 #' lines(cumhaz,col=2,lwd=2)
 #' 
-#' pctimecox <- pc.hazard(cumhaz,rrcox)
+#' pctimecox <- rchaz(cumhaz,rrcox)
 #' pctime <- cbind(pctime,X)
 #' ssx <- cox.aalen(Surv(time,status)~+prop(X),data=pctimecox,robust=0)
 #' plot(ssx)
@@ -45,12 +52,12 @@
 #' par(mfrow=c(1,2))
 #' plot(ss)
 #' ###
-#' pctime <- pc.hazard(ss$cum,1000)
+#' pctime <- rchaz(ss$cum,n=1000)
 #' ###
 #' sss <- aalen(Surv(time,status)~+1,data=pctime,robust=0)
 #' lines(sss$cum,col=2,lwd=2)
 #' 
-#' pctime <- pc.hazard(ss$cum,rrcox)
+#' pctime <- rchaz(ss$cum,rrcox)
 #' pctime <- cbind(pctime,X)
 #' ###
 #' sss <- cox.aalen(Surv(time,status)~+prop(X),data=pctime,robust=0)
@@ -58,9 +65,9 @@
 #' plot(ss)
 #' lines(sss$cum,col=3,lwd=3)
 #' 
-#' @export
-#' @aliases pchazard.sim addCums
-pc.hazard <- function(cumhazard,rr,n=NULL,entry=NULL,cum.hazard=TRUE,cause=1)
+#' @export 
+#' @aliases pc.hazard simrchaz addCums
+rchaz <- function(cumhazard,rr,n=NULL,entry=NULL,cum.hazard=TRUE,cause=1,extend=FALSE)
 {# {{{
 ### cumh=cbind(breaks,rates), first rate is 0 if cumh=FALSE
 ### cumh=cbind(breaks,cumhazard) if cumh=TRUE
@@ -75,7 +82,7 @@ pc.hazard <- function(cumhazard,rr,n=NULL,entry=NULL,cum.hazard=TRUE,cause=1)
   } else cumh <- cumhazard[,2] 
    n <- length(rr)
    ttt <- rexp(n)/rr
-   if (cumhazard[1,2]>0)  {
+   if (cumhazard[1,2]>0)  { ## start cumulative hazard with a 0
 	   warning("Safest to start with cumulative hazard 0 to avoid problems\n"); 
 	   cumhazard <- rbind(c(0,0),cumhazard)
 	   cumh <- c(0,cumh)
@@ -91,55 +98,31 @@ pc.hazard <- function(cumhazard,rr,n=NULL,entry=NULL,cum.hazard=TRUE,cause=1)
    rrx <- ifelse(rrx>mm,mm,rrx)
    status <- rep(0,n)
    status <- ifelse(rrx<mm,cause,status)
+   tcum <- tail(cumhazard,1)
+   extend.rate <- NULL
+   if (is.logical(extend))
+	   if (extend) extend.rate <- tcum[2]/tcum[1]
+   if (is.numeric(extend)) { extend.rate <- extend; extend <- TRUE;}
+   if (extend) {
+	   whoe <- which(status==0)
+	   nn <- length(whoe)
+	   if (nn>0) {
+	     textend <- rexp(nn)/(rr[whoe]*extend.rate)
+	     rrx[whoe] <- mm+textend
+	     status[whoe] <- 1
+	   }
+   }
    dt <- data.frame(entry=entry,time=rrx,status=status,rr=rr)
    colnames(dt) <- c("entry","time","status","rr"); 
    attr(dt,"cumhaz") <- cumhazard
+   attr(dt,"extend.rate") <- extend.rate
    return(dt)
 }# }}}
 
-###pc.hazard <- function(cumhazard,rr,entry=NULL,cum.hazard=TRUE,cause=1)
-###{# {{{
-###### cumh=cbind(breaks,rates), first rate is 0 if cumh=FALSE
-###### cumh=cbind(breaks,cumhazard) if cumh=TRUE
-###  if (length(rr)==1 & is.integer(rr)) rr<-rep(1,rr)
-###  breaks <- cumhazard[,1]
-###  rates <- cumhazard[,2][-1]
-###  mm <- tail(breaks,1)
-###  if (cum.hazard==FALSE) {
-###        cumh <- cumsum(c(0,diff(breaks)*rates))
-###        cumhazard <- cbind(breaks,cumh)
-###  } else cumh <- cumhazard[,2] 
-###   n <- length(rr)
-###   ttt <- rexp(n)/rr
-###   if (cumh[1]>0)  {
-###	   warning("Safest to start with cumulative hazard 0 to avoid problems\n"); 
-###	   cumh <- rbind(c(0,0),cumh)
-###   }
-###   ###
-###   if (!is.null(entry)) {
-###	   if (length(entry)==1) entry <- rep(entry,n) else entry <- entry
-###	   ## cumhazard at entry 
-###           ri <- sindex.prodlim(breaks,entry)
-###           rrr <- (entry-breaks[ri])/(breaks[ri+1]-breaks[ri])
-###           rrr[is.na(rrr)] <- 0
-###           cumentry <- rrr*(cumh[ri+1]-cumh[ri])+cumh[ri]
-###	   cumentry[is.na(cumentry)] <- tail(cumh,1)
-###   } else { entry <- cumentry <- rep(0,n) }
-###   ###
-###   ttte <- ttt+cumentry
-###   ri <- sindex.prodlim(cumh,ttte)
-###   rrr <- (ttte-cumh[ri])/(cumh[ri+1]-cumh[ri])
-###   rrr[is.na(rrr)] <- 0
-###   rrx <- rrr*(breaks[ri+1]-breaks[ri])+breaks[ri]
-###   rrx[is.na(rrx)] <- mm
-###   rrx <- ifelse(rrx>mm,mm,rrx)
-###   status <- rep(0,n)
-###   status <- ifelse(rrx<mm,cause,status)
-###   dt <- data.frame(entry=entry,time=rrx,status=status,rr=rr)
-###   colnames(dt) <- c("entry","time","status","rr"); 
-###   attr(dt,"cumhaz") <- cumhazard
-###   return(dt)
-###}# }}}
+pc.hazard <- function(cumhazard,rr,...)
+{# {{{
+rchaz(cumhazard,rr,...)
+}# }}}
 
 #' @export
 lin.approx <- function(x2,xfx,x=1)
@@ -159,7 +142,6 @@ lin.approx <- function(x2,xfx,x=1)
    return(res)
 }# }}}
 
-
 #' @export
 addCums <- function(cumB,cumA,max=NULL)
 {# {{{
@@ -172,25 +154,22 @@ addCums <- function(cumB,cumA,max=NULL)
  return(cbind(times,cumBA))
 }# }}}
 
-
-
 #' @export
-pchazard.sim <- function(cumhazard,rr,cens=NULL,rrc=NULL,cens.cum.hazard=TRUE,...)
+simrchaz <- function(cumhazard,rr,n=NULL,cens=NULL,rrc=NULL,...)
 {# {{{
-### cumh=cbind(breaks,rates), first rate is 0 if cumh=FALSE
-### cumh=cbind(breaks,cumhazard) if cumh=TRUE
+###   adds censoring to to rchaz call
+   if (!is.null(n)) rr <- rep(1,n)
+   n <- length(rr)  
 
-   ptt <- pc.hazard(cumhazard,rr,cum.hazard=TRUE,...)
+   ptt <- rchaz(cumhazard,rr,cum.hazard=TRUE,...)
 
-   if (length(rr)==1) n<-rr else n <- length(rr)
    if (is.null(rrc)) {
-	   if (length(rr)==1) rrc<-rr else rrc <- length(rr)
+	   rrc <- rep(1,n)
    }
    if (is.matrix(cens)) {
-	   pct <- pc.hazard(cumhazard,rr,cum.hazard=cens.cum.hazard,...)
+	   pct <- rchaz(cens,rrc,...)
 	   pct <- pct$time
-   }
-   else {
+   } else {
 	   if (is.numeric(cens)) pct<- rexp(n)/cens  else {
 	      chaz <-sum(ptt$status)/sum(ptt$time)  ## hazard averate T haz 
 	      pct<- rexp(n)/chaz 
@@ -204,16 +183,17 @@ pchazard.sim <- function(cumhazard,rr,cens=NULL,rrc=NULL,cens.cum.hazard=TRUE,..
 #' Simulation of Piecewise constant hazard models with two causes (Cox).
 #' 
 #' Simulates data from piecwise constant baseline hazard that can also be of
-#' Cox type. Censor data at highest value of the break points.
+#' Cox type. Censor data at highest value of the break points for either of the
+#' cumulatives.
 #' 
 #' @param cumhaz1 cumulative hazard of cause 1
 #' @param cumhaz2 cumulative hazard of cause 1
 #' @param rr1 number of simulations or vector of relative risk for simuations.
 #' @param rr2 number of simulations or vector of relative risk for simuations.
-#' @param cens to censor further 
+#' @param n number of simulation if rr not given 
+#' @param cens to censor further , rate or cumumlative hazard
 #' @param rrc retlativ risk for censoring.
-#' @param cens.cum.hazard possible cumulative hazard for censoring.
-#' @param ... arguments for pc.hazard 
+#' @param ... arguments for rchaz 
 #' @author Thomas Scheike
 #' @keywords survival
 #' @examples
@@ -234,7 +214,7 @@ pchazard.sim <- function(cumhazard,rr,cens=NULL,rrc=NULL,cens.cum.hazard=TRUE,..
 #'
 #' cumhaz1 <- cox1$cum
 #' cumhaz2 <- cox2$cum
-#' d <-  cause.pchazard.sim(cox1$cum,cox2$cum,rr1,rr2)
+#' d <-  rcrisk(cox1$cum,cox2$cum,rr1,rr2)
 #' dd <- cbind(d,Z1)
 #' sc1 <-   cox.aalen(Surv(time,status==1)~prop(vf)+prop(chf)+prop(wmi),
 #'                   data=dd,robust=0)
@@ -248,35 +228,41 @@ pchazard.sim <- function(cumhazard,rr,cens=NULL,rrc=NULL,cens.cum.hazard=TRUE,..
 #' lines(sc2$cum,col=2)
 #' 
 #' @export 
-cause.pchazard.sim<-function(cumhaz1,cumhaz2,rr1,rr2,cens=NULL,rrc=NULL,cens.cum.hazard=TRUE,...)
+#' @aliases cause.pchazard.sim 
+rcrisk <-function(cumhaz1,cumhaz2,rr1,rr2,n=NULL,cens=NULL,rrc=NULL,...)
 {#'# {{{
-##'## cumh=cbind(breaks,rates), first rate is 0 if cumh=FALSE
-##'## cumh=cbind(breaks,cumhazard) if cumh=TRUE
  
-   if (length(rr1)==1) n<-rr1 else n <- length(rr1)
-   if (missing(rr2)) rr2 <- n
- 
-   ptt1 <- pc.hazard(cumhaz1,rr1,cum.hazard=TRUE,...)
-   ptt2 <- pc.hazard(cumhaz2,rr2,cum.hazard=TRUE,...)
-   ptt <- data.frame(time=pmin(ptt1$time,ptt2$time),status=ifelse(ptt1$time<=ptt2$time,ptt1$status,ptt2$status*2))
- 
-   if (!is.null(cens)) {
-      if (is.null(rrc)) rrc <- n 
-           if (is.matrix(cens)) {
-        	   pct <- pc.hazard(cens,rrc,cum.hazard=cens.cum.hazard,...)
-        	   pct <- pct$time
-           } else {
-        	   if (is.numeric(cens)) pct<- rexp(n)/cens  else {
-        	      chaz <-sum(ptt1$status+ptt2$status)/sum(ptt1$time+ptt2$time)  ## hazard averate T haz 
-        	      pct<- rexp(n)/chaz 
-        	   }
-	   }
-	   ptt <- data.frame(time=pmin(ptt$time,pct), status=ifelse(ptt$time<pct,ptt$status,0))
-   }
+if (!is.null(n)) { rr1 <- rep(1,n); rr2 <- rep(1,n) }
+n <- length(rr1); 
+if (missing(rr2)) rr2 <-rep(1,n)
 
-   return(ptt)
+ptt1 <- rchaz(cumhaz1,rr1,cum.hazard=TRUE,...)
+ptt2 <- rchaz(cumhaz2,rr2,cum.hazard=TRUE,...)
+ptt <- data.frame(time=pmin(ptt1$time,ptt2$time),status=ifelse(ptt1$time<=ptt2$time,ptt1$status,ptt2$status*2),entry=ptt1$entry)
+
+if (!is.null(cens)) {
+	if (is.null(rrc)) rrc <- rep(1,n)
+	if (is.matrix(cens)) {
+		pct <- rchaz(cens,rrc,...)
+		pct <- pct$time
+	} else {
+		if (is.numeric(cens)) pct<- rexp(n)/cens  else {
+			chaz <-sum(ptt1$status+ptt2$status)/sum(ptt1$time+ptt2$time)  ## hazard averate T haz 
+			pct<- rexp(n)/chaz 
+		}
+	}
+	ptt <- data.frame(time=pmin(ptt$time,pct),status=ifelse(ptt$time<pct,ptt$status,0),entry=ptt$entry)
+}
+
+return(ptt)
 }# }}}
 
+#' @export 
+cause.pchazard.sim<-function(cumhaz1,cumhaz2,rr1,rr2,cens=NULL,rrc=NULL,...)
+{
+	rcrisk(cumhaz1,cumhaz2,rr1,rr2,cens=NULL,rrc=NULL,...)
+}
+
 #' Simulation of output from Cox model.
 #' 
 #' Simulates data that looks like fit from Cox model. Censor data automatically
@@ -291,13 +277,12 @@ cause.pchazard.sim<-function(cumhaz1,cumhaz2,rr1,rr2,cens=NULL,rrc=NULL,cens.cum
 #' given then takes average rate of in simulated data from cox model.
 #' @param rrc possible vector of relative risk for cox-type censoring.
 #' @param entry delayed entry variable for simulation.
-#' @param ... arguments for pc.hazard, for example entry-time
+#' @param ... arguments for rchaz, for example entry-time
 #' @author Thomas Scheike
 #' @keywords survival
 #' @examples
 #' 
 #' data(TRACE)
-#' 
 #' cox <-  coxph(Surv(time,status==9)~vf+chf+wmi,data=TRACE)
 #' sim1 <- sim.cox(cox,1000,data=TRACE)
 #' cc <- coxph(Surv(time,status)~vf+chf+wmi,data=sim1)
@@ -306,14 +291,6 @@ cause.pchazard.sim<-function(cumhaz1,cumhaz2,rr1,rr2,cens=NULL,rrc=NULL,cens.cum
 #' cor(sim1[,c("vf","chf","wmi")])
 #' cor(TRACE[,c("vf","chf","wmi")])
 #' 
-#' cox <-  cox.aalen(Surv(time, status==9) ~ prop(vf)+prop(chf)+prop(wmi),TRACE,robust=0)
-#' sim2 <- sim.cox(cox,1000,data=TRACE)
-#' cc <-  cox.aalen(Surv(time, status)~prop(vf)+prop(chf)+prop(wmi),data=sim2,robust=0)
-#' ###
-#' plot(cox)
-#' lines(cc$cum,type="s",col=2)
-#' cbind(cox$gamma,cc$gamma)
-#' 
 #' \donttest{
 #' ### do not test to avoid dependence on mets
 #' library(mets)
@@ -336,53 +313,53 @@ cause.pchazard.sim<-function(cumhaz1,cumhaz2,rr1,rr2,cens=NULL,rrc=NULL,cens.cum
 #' @aliases sim.cox read.fit 
 sim.cox <- function(cox,n,data=NULL,cens=NULL,rrc=NULL,entry=NULL,...)
 {# {{{
-### cumh=cbind(breaks,rates), first rate is 0 if cumh=FALSE
-### cumh=cbind(breaks,cumhazard) if cumh=TRUE
-des <- 	read.fit(cox,n,data=data,...)
-Z <- des$Z; cumhazard <- des$cum; rr <- des$rr; 
+	### cumh=cbind(breaks,rates), first rate is 0 if cumh=FALSE
+	### cumh=cbind(breaks,cumhazard) if cumh=TRUE
+	des <- 	read.fit(cox,n,data=data,...)
+	Z <- des$Z; cumhazard <- des$cum; rr <- des$rr; 
 
-if (class(cox)!="phreg") {
-        if (cumhazard[1,2]>0) cumhazard <- rbind(c(0,0),cumhazard)
-	ptt <- pc.hazard(cumhazard,rr,entry=entry) 
-	ptt <- cbind(ptt,Z)
-} else {
-	ptt <- data.frame()
-	stratj <- cox$strata[cox$jumps]
-	if (cox$nstrata>1) {
-		for (j in 0:(cox$nstrata-1)) {
-                        cumhazardj <- rbind(c(0,0),cox$cumhaz[stratj==j,])
-			if (!is.null(entry)) entry <- entry[des$strataid==j]
-	                pttj <- pc.hazard(cumhazardj,rr[des$strataid==j],entry=entry) 
-			Zj <- Z[des$strataid==j,,drop=FALSE]
-			pttj <- cbind(pttj,Zj)
-			ptt  <-  rbind(ptt,pttj)
-		}
-	} else {
-            if (cumhazard[1,2]>0) cumhazard <- rbind(c(0,0),cumhazard)
-		ptt <- pc.hazard(cumhazard,rr,entry=entry) 
+	if (class(cox)!="phreg") {
+		if (cumhazard[1,2]>0) cumhazard <- rbind(c(0,0),cumhazard)
+		ptt <- rchaz(cumhazard,rr,entry=entry) 
 		ptt <- cbind(ptt,Z)
+	} else {
+		ptt <- data.frame()
+		stratj <- cox$strata[cox$jumps]
+		if (cox$nstrata>1) {
+			for (j in 0:(cox$nstrata-1)) {
+				cumhazardj <- rbind(c(0,0),cox$cumhaz[stratj==j,])
+				if (!is.null(entry)) entry <- entry[des$strataid==j]
+				pttj <- rchaz(cumhazardj,rr[des$strataid==j],entry=entry) 
+				Zj <- Z[des$strataid==j,,drop=FALSE]
+				pttj <- cbind(pttj,Zj)
+				ptt  <-  rbind(ptt,pttj)
+			}
+		} else {
+			if (cumhazard[1,2]>0) cumhazard <- rbind(c(0,0),cumhazard)
+			ptt <- rchaz(cumhazard,rr,entry=entry) 
+			ptt <- cbind(ptt,Z)
+		}
 	}
-}
 
-   if (!is.null(cens))  {
-      if (is.null(rrc)) rrc <- rep(1,n)
-      if (is.matrix(cens)) {
-	   pct <- pc.hazard(cens,rrc,cum.hazard=TRUE,entry=entry)
-	   pct <- pct$time
-      }
-      else {
-	   if (is.numeric(cens)) pct<- rexp(n)/cens  else {
-	      chaz <-sum(ptt$status)/sum(ptt$time)  ## hazard averate T haz 
-	      pct<- rexp(n)/chaz 
-              if (!is.null(entry)) pct  <- entry + pct
-           }
-      }
-      ptt$time <- pmin(ptt$time,pct)
-      ptt$status <- ifelse(ptt$time<pct,ptt$status,0)
-   } 
+	if (!is.null(cens))  {
+		if (is.null(rrc)) rrc <- rep(1,n)
+		if (is.matrix(cens)) {
+			pct <- rchaz(cens,rrc,entry=entry)
+			pct <- pct$time
+		}
+		else {
+			if (is.numeric(cens)) pct<- rexp(n)/cens  else {
+				chaz <-sum(ptt$status)/sum(ptt$time)  ## hazard averate T haz 
+				pct<- rexp(n)/chaz 
+				if (!is.null(entry)) pct  <- entry + pct
+			}
+		}
+		ptt$time <- pmin(ptt$time,pct)
+		ptt$status <- ifelse(ptt$time<pct,ptt$status,0)
+	} 
 
-   attr(ptt,"id") <- des$id
-   return(ptt)
+	attr(ptt,"id") <- des$id
+	return(ptt)
 }# }}}
 
 #' Simulation of cause specific from Cox models.
@@ -404,28 +381,23 @@ if (class(cox)!="phreg") {
 #'             given then takes average rate of in simulated data from cox model.
 #'             But censoring can also be given as a cause.
 #' @param rrc possible vector of relative risk for cox-type censoring.
-#' @param ... arguments for pc.hazard, for example entry-time
+#' @param ... arguments for rchaz, for example entry-time
 #' @author Thomas Scheike
 #' @keywords survival
 #' @examples
+#' nsim <- 100
 #' 
-#' nsim <- 1000
 #' data(bmt)
-#' cox1 <- cox.aalen(Surv(time,cause==1)~prop(tcell)+prop(platelet),data=bmt,robust=0)
-#' cox2 <- cox.aalen(Surv(time,cause==2)~prop(tcell)+prop(platelet),data=bmt,robust=0)
+#' # coxph          
+#' cox1 <- coxph(Surv(time,cause==1)~tcell+platelet,data=bmt)
+#' cox2 <- coxph(Surv(time,cause==2)~tcell+platelet,data=bmt)
 #' coxs <- list(cox1,cox2)
 #' dd <- sim.cause.cox(coxs,nsim,data=bmt)
-#' scox1 <- cox.aalen(Surv(time,status==1)~prop(tcell)+prop(platelet),data=dd,robust=0)
-#' scox2 <- cox.aalen(Surv(time,status==2)~prop(tcell)+prop(platelet),data=dd,robust=0)
-#' ### 
-#' cbind(cox1$gamma,scox1$gamma)
-#' cbind(cox2$gamma,scox2$gamma)
-#' par(mfrow=c(1,2))
-#' plot(cox1); lines(scox1$cum,col=2)
-#' plot(cox2$cum,type="l");
-#' lines(scox2$cum,col=2)
-#'           
-#'           
+#' scox1 <- coxph(Surv(time,status==1)~tcell+platelet,data=dd)
+#' scox2 <- coxph(Surv(time,status==2)~tcell+platelet,data=dd)
+#' cbind(cox1$coef,scox1$coef)
+#' cbind(cox2$coef,scox2$coef)
+#' 
 #' \donttest{
 #' ### do not test to avoid dependence on mets
 #' library(mets)          
@@ -454,15 +426,6 @@ if (class(cox)!="phreg") {
 #' basehazplot.phreg(cox1); basehazplot.phreg(scox1,add=TRUE); 
 #' basehazplot.phreg(cox2); basehazplot.phreg(scox2,add=TRUE); 
 #' 
-#' # coxph          
-#' cox1 <- coxph(Surv(time,cause==1)~tcell+platelet,data=bmt)
-#' cox2 <- coxph(Surv(time,cause==2)~tcell+platelet,data=bmt)
-#' coxs <- list(cox1,cox2)
-#' dd <- sim.cause.cox(coxs,nsim,data=bmt)
-#' scox1 <- coxph(Surv(time,status==1)~tcell+platelet,data=dd)
-#' scox2 <- coxph(Surv(time,status==2)~tcell+platelet,data=dd)
-#' cbind(cox1$coef,scox1$coef)
-#' cbind(cox2$coef,scox2$coef)
 #' }  
 #' @export
 sim.cause.cox <- function(coxs,n,data=NULL,cens=NULL,rrc=NULL,...)
@@ -492,7 +455,7 @@ if (!is.list(coxs)) stop("Cox models in list form\n");
 
    if (!is.null(cens)) {
    if (is.matrix(cens)) {
-	   pct <- pc.hazard(cens,rrc,cum.hazard=TRUE)
+	   pct <- rchaz(cens,rrc)
 	   pct <- pct$time
    }
    else {
@@ -502,8 +465,7 @@ if (!is.list(coxs)) stop("Cox models in list form\n");
 	   }
    }
 
-   dt <- cbind(data.frame(time=pmin(ptt$time,pct),
-			  status=ifelse(ptt$time<pct,ptt$status,0)),simcovs)
+   dt <- cbind(data.frame(time=pmin(ptt$time,pct),status=ifelse(ptt$time<pct,ptt$status,0)),simcovs)
    }
 
    return(dt)
@@ -516,6 +478,7 @@ if (!is.list(coxs)) stop("Cox models in list form\n");
 #' 
 #' @param cumhazard matrix that specified times and values of some cumulative hazard.
 #' @param rr "relative risk" terms 
+#' @param n number of simulation if rr not given 
 #' @param entry not implemented yet 
 #' @param type either cloglog or logistic 
 #' @param startcum c(0,0) to make cumulativ start in 0 with 0 cumhazard.
@@ -529,7 +492,7 @@ if (!is.list(coxs)) stop("Cox models in list form\n");
 #' cumhaz <- cif$cum
 #' 
 #' ## 1000 logistic without covariates with baseline from model fit  
-#' sim1 <- simsubdist(cumhaz,1000,type="logistic")
+#' sim1 <- simsubdist(cumhaz,n=1000,type="logistic")
 #' ###
 #' cifs <- comp.risk(Event(time,status)~+1,data=sim1,cause=1,model="logistic2")
 #' ###
@@ -557,7 +520,7 @@ if (!is.list(coxs)) stop("Cox models in list form\n");
 #' F1p<-function(t,lam0=0.4,beta=0.3,x=0) # proportional version 
 #' { return( 1 - exp(-(t*lam0)*exp(x*beta))) }
 #' 
-#' n=10000
+#' n=1000
 #' tt=seq(0,3,by=.01)
 #' tt=seq(0,3,by=.01)
 #' t1 <- invsubdist(cbind(tt,F1p(tt)),runif(n))
@@ -590,13 +553,15 @@ if (!is.list(coxs)) stop("Cox models in list form\n");
 #' #lines(tt,F1p(tt,lam0=0.1),col=2)
 #' @aliases simsubdist
 #' @export
-simsubdist <- function(cumhazard,rr,entry=NULL,type="cloglog",startcum=c(0,0),...)
+simsubdist <- function(cumhazard,rr,n=NULL,entry=NULL,type="cloglog",startcum=c(0,0),...)
 {# {{{
   ## Fine-Gray model cloglog F1= 1-exp(-cum(t)*rr)
   ## logistic                F1= cum(t)*rr/(1+cum(t)*rr)
+  ## identity                F1= cum(t)*rr
   ## rr=exp(X^t beta) 
+  if (!is.null(n)) rr <- rep(1,n)
   entry=NULL
-  if (length(rr)==1) rr<-rep(1,rr)
+
   breaks <- cumhazard[,1]
   rates <- cumhazard[,2][-1]
   mm <- tail(breaks,1)
@@ -609,25 +574,27 @@ simsubdist <- function(cumhazard,rr,entry=NULL,type="cloglog",startcum=c(0,0),..
   }
   if (type=="cloglog") {
       F1tau <- 1-exp(-tail(cumh,1)*rr)
-      rb <- rbinom(n,1,F1tau)
       ttt <- -log(1-runif(n)*F1tau)/rr
   } else if (type=="logistic") {
      F1tau <- tail(cumh,1)*rr/(1+tail(cumh,1)*rr)
      v <- runif(n)*F1tau
      ttt <- exp(logit(v))/rr; 
+  }  else if (type=="identity" | type=="cif") {
+     F1tau <- tail(cumh,1)
+     ttt <- runif(n)*F1tau
+     ## rr only affects binomial draw 
+     F1tau <- F1tau*rr 
   } else stop(" cloglog or logistic or give function (fun=) \n"); 
   ###
    entry <- cumentry <- rep(0,n)
    ttte <- ttt+cumentry
-   ri <- sindex.prodlim(cumh,ttte)
-   rrr <- (ttte-cumh[ri])/(cumh[ri+1]-cumh[ri])
-   rrx <- rrr*(breaks[ri+1]-breaks[ri])+breaks[ri]
-   rrx[is.na(rrx)] <- mm
-   rrxo <- rrx
-   status <- rbinom(n,1,F1tau)
+   rrx <- lin.approx(ttt,cbind(breaks,cumh),x=-1)
+   timecause <- rrx
+   ###
+   rrx <- ifelse(rrx>mm,mm,rrx)
+   status <- rbinom(n,1,F1tau) 
    rrx[status==0] <- mm
-   dt <- data.frame(entry=entry,time=rrx,status=status,rr=rr,
-		    F1tau=F1tau,timecause=rrxo)
+   dt <- data.frame(entry=entry,time=rrx,status=status,rr=rr,F1tau=F1tau,timecause=timecause)
    attr(dt,"cumhaz") <- cumhazard
    return(dt)
 }# }}}
@@ -639,8 +606,7 @@ simsubdist <- function(cumhazard,rr,entry=NULL,type="cloglog",startcum=c(0,0),..
 #' @param F1 matrix with x, and F1(x) 
 #' @param u points for which to compute inverse
 #' @param entry possible delayed entry points
-#' @param cond 1 indcates that we draw given that this subdistribution is used, so scales mass to 1 to get conditional 
-#'               distribution function
+#' @param cond 1 indcates that we draw given that this subdistribution is used, so draws from F1(t)/F1(tau) 
 #' @param ptrunc possible trunction weigth for delayed entry, if NULL then uses ptrunc=1-F1(entry)
 #' @author Thomas Scheike
 #' @keywords survival
@@ -660,36 +626,6 @@ simsubdist <- function(cumhazard,rr,entry=NULL,type="cloglog",startcum=c(0,0),..
 #' Fiu <- invsubdist(F1,u,cond=1)
 #' points(Fiu$time,u,pch="x")
 #' 
-#' entry <- 4
-#' ###
-#' F1entry <- subdist(F1,entry)[,2]
-#' ptrunc <- 1-F1entry
-#' ###
-#' F1entry5 <- F1
-#' F1entry5[,1] <- F1entry5[,1]-entry
-#' F1entry5[,2] <- (F1entry5[,2]-F1entry)/ptrunc
-#' pos <- F1entry5[,1]>=0
-#' F1entry5 <- rbind(c(0,0),F1entry5[pos,])
-#' ###
-#' plot(F1entry5,ylim=c(0,1),type="l")
-#' u <- runif(100)
-#' Fiu <- invsubdist(F1entry5,u,cond=0)
-#' points(Fiu$time,u,pch="-")
-#' ###
-#' Fiu2 <- invsubdist(F1,u,cond=0,entry=entry)
-#' points(Fiu2$time-entry,u,pch="x")
-#' sum(Fiu2$time-entry-Fiu$time)
-#' 
-#' F1ce <- F1entry5
-#' F1ce[,2] <- F1ce[,2]/tail(F1entry5[,2],1)
-#' plot(F1ce,type="l")
-#' u <- runif(100)
-#' Fi1ce <- invsubdist(F1ce,u,cond=0)
-#' points(Fi1ce$time,u,pch="-")
-#' Fice <- invsubdist(F1,u,cond=1,entry=entry)
-#' points(Fice$time-entry,u,pch="x")
-#' sum(Fice$time-entry-Fi1ce$time)
-#' 
 #' 
 #' ## simulation of distribution with delayed entry starting at 3
 #' par(mfrow=c(1,1))
@@ -780,48 +716,30 @@ simsubdist <- function(cumhazard,rr,entry=NULL,type="cloglog",startcum=c(0,0),..
 #' @export
 invsubdist <- function(F1,u,entry=NULL,cond=1,ptrunc=NULL)
 {# {{{
-  x <- breaks <- F1[,1]
-  y <- F1t <- F1[,2] 
-  mm <- tail(breaks,1)
-  F1m <- tail(F1t,1)
-  n <- length(u)
-  if (!is.null(entry)) {
-      ###  if (length(entry)==1) entry <- rep(entry,n) else entry <- entry
-      ##   cumhazard at entry 
-      F1entry <- subdist(F1,entry)[,2]; 
-  } 
-  if (!is.null(entry)) 
-	  if (is.null(ptrunc)) ptrunc <- 1- F1entry
-  if (cond==0 & !is.null(entry))  u<-ptrunc*u+F1entry
-  if (cond==1 & !is.null(entry))  u<-u*(F1m-F1entry)+F1entry 
-###  if (cond==0 & is.null(entry))   u<-u
-  if (cond==1 & is.null(entry))   u<-u*F1m
-  y <- F1t
-  ri <- sindex.prodlim(y,u,strict=FALSE)
-  rrr <- (u-y[ri])/(y[ri+1]-y[ri])
-  rrx <- rrr*(x[ri+1]-x[ri])+x[ri]
+  ### computes inverse subdist F1^{-1}(u) 
+  ### computes inverse subdist F1^{-1}(u)
+  F1max <- tail(F1[,2],1)
+  mmax <- tail(F1[,1],1)
+
+  if (!is.null(entry))  F1entry <- subdist(F1,entry)[,2];  
+  if (!is.null(entry)) if (is.null(ptrunc)) ptrunc <- 1- F1entry
+  if (cond==0 & !is.null(entry))  u<-u*ptrunc+F1entry
+  if (cond==1 & !is.null(entry))  u<-u*(F1max-F1entry)+F1entry 
+  if (cond==1 & is.null(entry))   u<-u*F1max
+  rrx  <- lin.approx(u,F1,x=-1)
   status <- rep(1,length(u))
-  status[is.na(rrx)] <- 0
-  rrx[is.na(rrx)] <- mm
+  status[u>=F1max] <- 0
   dt <- data.frame(time=rrx,status=status,y=u)
   if (!is.null(entry)) dt <- cbind(dt,entry)
   attr(dt,"F1") <- F1
-  attr(dt,"Fmax") <- F1m
+  attr(dt,"Fmax") <- F1max
   return(dt)
 }# }}}
 
-
 #' @export
 subdist <- function(F1,times)
 {# {{{
-  x <- breaks <- F1[,1]
-  y <- F1t <- F1[,2] 
-  mm <- tail(breaks,1)
-  F1m <- tail(F1t,1)
-  ri <- sindex.prodlim(x,times,strict=FALSE)
-  rrr <- (times-x[ri])/(x[ri+1]-x[ri])
-  rrx <- rrr*(y[ri+1]-y[ri])+y[ri]
-  rrx[is.na(rrx)] <- F1m 
+  rrx <-   lin.approx(times,F1,x=1)
   dt <- cbind(times,rrx)
   colnames(dt) <- c("time","subdist")
   return(dt)
@@ -875,15 +793,13 @@ subdist <- function(F1,times)
 #' plot(cif) 
 #' lines(cc$cum)
 #' 
-#' # data(TRACE)
-#' #  mm <- model.matrix(~vf+chf+wmi,data=TRACE)[,-1]
-#' #  library(cmprsk)
-#' #  cif <-  crr(TRACE$time,TRACE$status,mm,failcode=9)
-#' #  sim1 <- sim.cif(cif,10000,data=TRACE,Z=mm)
-#' #  mms <- model.matrix(~vf+chf+wmi,data=sim1)[,-1]
-#' #  #' cc <-  prop.odds.subdist(Event(time,status)~vf+chf+wmi,data=sim1,cause=1)
-#' #  cif1 <-  crr(sim1$time,sim1$status,mms,failcode=1)
-#' #  cbind(cif$coef,cif1$coef)
+#' # faster/better with  mets package 
+#' # library(mets)
+#' # scif <-  cifreg(Event(time,status)~vf+chf+wmi,data=sim1,cause=1,prop=NULL)
+#' #   
+#' # plot(scif$cum,type="l")
+#' # lines(cif$cum,col=2)
+#' # cbind(cif$gamma,scif$coef)
 #' # 
 #' ################################################################
 #' #  simulating several causes with specific cumulatives 
@@ -924,7 +840,7 @@ subdist <- function(F1,times)
 #'  lines(cifs[[1]]$cum,col=2)
 #'  lines(cifs[[2]]$cum,col=2,lty=2)
 #'
-#' #  Everyhing wraped in a call assuming covariates work in the same way for two models
+#' #  Everyhing wrapped in call assuming covariates work in the same way for two models
 #' dd <- sim.cifs(list(cif1,cif2),2000,data=bmt)
 #' scif1 <-  comp.risk(Event(time,cause)~const(tcell)+const(age),
 #'                   data=dd,cause=1,model="logistic2")
@@ -937,43 +853,70 @@ subdist <- function(F1,times)
 #' lines(cifs[[1]]$cum,col=2)
 #' lines(cifs[[2]]$cum,col=2,lty=2)
 #'
+#' # Everyhing wrapped in call assuming covariates work in the same way for two models
+#' # but now draws cif1 to be of correct model, but model 2 is adapted 
+#' #(if needed) to make constraints satisfied F1+F2 <=1
+#' # see doubleFG of mets package for paramtrization
+#' # and drawns as "if not cause1" then distribute according to cause 2
+#' # dd <- sim.cifsRestrict(list(cif1,cif2),2000,data=bmt)
+#
+#' # faster with mets package 
+#' # dd <- sim.cifs(list(cif1,cif2),1000,data=bmt)
+#' # scif1 <-  cifreg(Event(time,cause)~tcell+age,data=dd,cause=1)
+#' # scif2 <-  cifreg(Event(time,cause)~tcell+age,data=dd,cause=2)
+#' #   
+#' # plot(scif1$cum,type="l")
+#' # legend("topleft",c("cause1","cause2"),lty=1:2,col=1:1)
+#' # lines(cifs[[1]]$cum,col=2)
+#' # lines(cifs[[2]]$cum,col=2,lty=2)
+#' # 
 #' @export
-#' @aliases sim.cif sim.cifs subdist pre.cifs 
-sim.cif <- function(cif,n,data=NULL,Z=NULL,drawZ=TRUE,cens=NULL,rrc=NULL,
-		    cumstart=c(0,0),...)
+#' @aliases sim.cif sim.cifs subdist pre.cifs sim.cifsRestrict setup.cif
+sim.cif <- function(cif,n,data=NULL,Z=NULL,drawZ=TRUE,cens=NULL,rrc=NULL,cumstart=c(0,0),...)
 {# {{{
-### cumh=cbind(breaks,rates), first rate is 0 if cumh=FALSE
-### cumh=cbind(breaks,cumhazard) if cumh=TRUE
-
 ## also extracts coefficients and baseline from coxph, cox.aalen, phreg
 ## and uses them assuming that the model is cloglog unless otherwise
+if (class(cif)[1]!="defined")  {
 des <- 	read.fit(cif,n,data=data,Z=Z,drawZ=drawZ,...)
-Z <- des$Z; cumhazard <- des$cum; rr <- des$rr; 
-cumhazard <- rbind(cumstart,cumhazard)
+Z <- des$Z; cumhaz <- des$cum; rr <- des$rr; 
+id <- des$id
+cumhaz <- rbind(cumstart,cumhaz)
 znames <- names(Z); 
 model <- des$model
-
-if (class(cif)!="phreg") {
-if (model=="logistic2" | model=="logistic") ptt <- simsubdist(cumhazard,rr,type="logistic",...) else ptt <- simsubdist(cumhazard,rr,type="cloglog",...)
-    ptt <- cbind(ptt,Z)
 } else {
+cumhaz <- cif$cumhaz
+if (is.null(Z)) stop("must give Z")
+rr <- exp(as.matrix(Z) %*% cif$coef)
+if (is.data.frame(Z)) znames <- names(Z) else znames <- colnames(Z)
+model <- cif$model
+id <- 1:nrow(Z)
+}
+
+if (class(cif)[1]!="phreg") {
+if (model=="logistic2" | model=="logistic") ptt <- simsubdist(cumhaz,rr,type="logistic",...) else ptt <- simsubdist(cumhaz,rr,type="cloglog",...)
+    ptt <- cbind(ptt,Z)
+} else { ### phreg class# {{{
 	ptt <- data.frame()
 	if (cif$nstrata>1) {
 		stratj <- cif$strata[cif$jumps]
 		for (j in 0:(cif$nstrata-1)) {
                         cumhazardj <- rbind(c(0,0),cif$cumhaz[stratj==j,])
-	                pttj <- pc.hazard(cumhazardj,rr[des$strataid==j]) 
+			if (model[3]) 
+		          pttj <- simsubdist(cumhazardj,rr[des$strataid==j],type="cloglog") else 
+	                  pttj <- simsubdist(cumhazardj,rr[des$strataid==j],type="logistic") 
 			Zj <- Z[des$strataid==j,,drop=FALSE]
 			pttj <- cbind(pttj,Zj)
 			ptt  <-  rbind(ptt,pttj)
 			ptt  <-  rbind(ptt,pttj)
 		}
 	} else {
-		ptt <- pc.hazard(cumhazard,rr,...) 
+	if (model[3]) ptt <- simsubdist(cumhaz,rr,type="cloglog") else 
+	              ptt <- simsubdist(cumhaz,rr,type="logistic") 
 		ptt <- cbind(ptt,Z)
 	}
- }
+ }# }}}
 
+  ### adds censoring 
    if (!is.null(cens))  {# {{{
       if (is.null(rrc)) rrc <- rep(1,n)
       if (is.matrix(cens)) {
@@ -991,7 +934,7 @@ if (model=="logistic2" | model=="logistic") ptt <- simsubdist(cumhazard,rr,type=
    } # }}}
 
    attr(ptt,"model") <- model
-   attr(ptt,"id") <-  des$id
+   attr(ptt,"id") <-  id
    attr(ptt,"znames") <- znames
    return(ptt)
 }# }}}
@@ -1004,24 +947,85 @@ if (!is.list(cifs)) stop("Cif models in list form\n");
 
   ## must consider all models out to last observation times or max.times
   cifs <- pre.cifs(cifs,max.times=max.times)
+
   tau <- tail(cifs[[1]]$cum[,1],1)
   sim1 <- sim.cif(cifs[[1]],n,data=data,Z=Z)
   Z <- sim1[,attr(sim1,"znames")]
-  ptot <- sim1$F1tau
-  sim2p <- read.fit(cifs[[2]],1,data=data,Z=NULL)
-  Z2 <- data[attr(sim1,"id"),names(sim2p$Z)]
-  sim2 <- sim.cif(cifs[[2]],n,data=data,Z=Z2,drawZ=FALSE)
+  if (class(cifs[[2]])!="defined")  {
+      sim2p <- read.fit(cifs[[2]],1,data=data,Z=NULL)
+      Z2 <- data[attr(sim1,"id"),names(sim2p$Z)]
+      sim2 <- sim.cif(cifs[[2]],n,data=data,Z=Z2,drawZ=FALSE)
+  } else { 
+        Z2 <- Z[,attr(cifs[[2]],"znames")]
+	sim2 <- sim.cif(cifs[[2]],n,data=data,Z=Z2)
+  }
 
-  ptot <- ptot+sim2$F1tau
+  ptot <- sim1$F1tau+sim2$F1tau
   ###
   rt <- rbinom(n,1,pmin(ptot,1))
   rb <- rbinom(n,1,sim1$F1tau/ptot)
-  cause=ifelse(rb==1,causes[1],causes[2])
+  cause=ifelse(rb==1,1,2)
   time=ifelse(cause==causes[1],sim1$timecause,sim2$timecause)
   cause <- rt*cause
   time[cause==0] <- tau
 
-  ptt <- data.frame(time=time,cause=cause,ptot=ptot)
+  ptt <- data.frame(time=time,status=cause,cause=cause,ptot=ptot)
+  Zcovs <- cbind(Z,Z2)
+  samecovs <-  match(names(Z2),names(Z)) 
+  Ze <- Zcovs[,-samecovs]
+  ptt <- cbind(ptt,Ze)
+
+   if (!is.null(cens))  {# {{{
+      if (is.null(rrc)) rrc <- rep(1,n)
+      if (is.matrix(cens)) {
+	   pct <- pc.hazard(cens,rrc,cum.hazard=TRUE,...)
+	   pct <- pct$time
+      }
+      else {
+	   if (is.numeric(cens)) pct<- rexp(n)/cens  else {
+	      chaz <-sum(ptt$status)/sum(ptt$time)  ## hazard averate T haz 
+	      pct<- rexp(n)/chaz 
+           }
+      }
+      ptt$time <- pmin(ptt$time,pct)
+      ptt$status <- ifelse(ptt$time<pct,ptt$status,0)
+   } # }}}
+
+   return(ptt)
+}# }}}
+
+#' @export
+sim.cifsRestrict <- function(cifs,n,data=NULL,Z=NULL,cens=NULL,rrc=NULL,max.times=NULL,causes=c(1,2),...)
+{# {{{
+
+if (!is.list(cifs)) stop("Cif models in list form\n"); 
+
+  ## must consider all models out to last observation times or max.times
+  cifs <- pre.cifs(cifs,max.times=max.times)
+
+  tau <- tail(cifs[[1]]$cum[,1],1)
+  sim1 <- sim.cif(cifs[[1]],n,data=data,Z=Z)
+  Z <- sim1[,attr(sim1,"znames")]
+  if (class(cifs[[2]])[1]!="defined")  {
+      sim2p <- read.fit(cifs[[2]],1,data=data,Z=NULL)
+      Z2 <- data[attr(sim1,"id"),names(sim2p$Z)]
+      sim2 <- sim.cif(cifs[[2]],n,data=data,Z=Z2,drawZ=FALSE)
+  } else { 
+        Z2 <- Z[,attr(cifs[[2]],"znames")]
+	sim2 <- sim.cif(cifs[[2]],n,data=data,Z=Z2)
+  }
+
+
+  ptot <- sim1$F1tau+sim2$F1tau*(1-sim1$F1tau)
+  ###
+  rt <- rbinom(n,1,pmin(ptot,1))
+  rb <- rbinom(n,1,sim1$F1tau/ptot)
+  cause=ifelse(rb==1,1,2)
+  time=ifelse(cause==causes[1],sim1$timecause,sim2$timecause)
+  cause <- rt*cause
+  time[cause==0] <- tau
+
+  ptt <- data.frame(time=time,status=cause,cause=cause,ptot=ptot)
   ptt <- cbind(ptt,Z)
   samecovs <-  match(names(Z2),names(Z)) 
   Ze <- Z2[,-samecovs]
@@ -1030,6 +1034,7 @@ if (!is.list(cifs)) stop("Cif models in list form\n");
    if (!is.null(cens))  {# {{{
       if (is.null(rrc)) rrc <- rep(1,n)
       if (is.matrix(cens)) {
+           cum.hazard <- TRUE
 	   pct <- pc.hazard(cens,rrc,cum.hazard=TRUE,...)
 	   pct <- pct$time
       }
@@ -1056,13 +1061,16 @@ if (!is.list(cifs)) stop("Cif models in list form\n");
 ## extracts maximum time of all cumulatives
 maxtimes <- rep(0,length(cifs))
   for (i in 1:length(cifs)) { 
-     if (class(cifs[[i]])=="coxph") {
+     if (class(cifs[[i]][1])=="coxph") {
 	     stop("Use phreg of mets instead\n"); 
-     } else  if (class(cifs[[i]])=="crr") {
+     } else  if (class(cifs[[i]])[1]=="crr") {
 	cifs[[i]]$cum <- rbind(c(0,0),cbind(cifs[[i]]$uftime,cumsum(cifs[[i]]$bfitj)))
         maxtimes[i] <- max(cifs[[i]]$uftime)
-     } else if (class(cifs[[i]])=="phreg") {
-	cifs[[i]]$cum <- cifs[[i]]$cumhaz
+     } else if (class(cifs[[i]])[1]=="phreg") {
+	cifs[[i]]$cumhaz <- cifs[[i]]$cumhaz
+        maxtimes[i] <- max(cifs[[i]]$cumhaz[,1])
+     }  else if (class(cifs[[i]])[1]=="defined") {
+	cifs[[i]]$cumhaz <- cifs[[i]]$cumhaz
         maxtimes[i] <- max(cifs[[i]]$cumhaz[,1])
      } else {
         maxtimes[i] <- max(cifs[[i]]$cum[,1])
@@ -1084,6 +1092,7 @@ maxtimes <- rep(0,length(cifs))
 	cums <- cums[keep,,drop=FALSE]
 	cums <- rbind(cums,Fmm)
 	cifs[[i]]$cum <- cums
+	cifs[[i]]$cumhaz <- cums
 	if (pprint) {
 	print(head(cums))
 	print(tail(cums))
@@ -1093,7 +1102,20 @@ maxtimes <- rep(0,length(cifs))
   return(cifs)
 }# }}}
 
-## reads a coxph, cox.aale, phreg, crr, comprisk, prop.odds.subdist object
+#' @export 
+setup.cif  <- function(cumhazard,coef,Znames=NULL,type="logistic")
+{# {{{
+cif <- list()
+cif$cumhaz <- cumhazard
+cif$coef <- coef
+cif$model <- type
+class(cif) <- "defined"
+attr(cif,"znames") <- Znames
+return(cif)
+}# }}}
+
+
+## reads a coxph, cox.aalen, phreg, crr, comprisk, prop.odds.subdist object
 ## and returns cumulative hazard, linear predictor of a resample of size
 ## n of data, for coxph cox.aalen, comprisk the design Z is constructed
 ## using the data, but Z can also be given which is needed for crr
@@ -1103,8 +1125,7 @@ maxtimes <- rep(0,length(cifs))
 read.fit <- function(cox,n,data=NULL,Z=NULL,drawZ=TRUE,fixZ=FALSE,id=NULL)
 {# {{{
 
-
-if (class(cox)=="coxph")
+if (class(cox)[1]=="coxph")
 {# {{{
    base <- basehaz(cox,centered=FALSE)[,c(2,1)] 
    mt <- model.frame(cox)
@@ -1129,7 +1150,7 @@ if (class(cox)=="coxph")
   colnames(Z) <- nn
   model <- "fg"
 }# }}}
-if (class(cox)=="cox.aalen")
+if (class(cox)[1]=="cox.aalen")
 {# {{{
    formula <- attr(cox, "Formula")
 ###   Z1 <- na.omit(model.matrix(cox,data=data))
@@ -1149,7 +1170,7 @@ if (class(cox)=="cox.aalen")
    model <- "fg"
    if (cox$prop.odds==TRUE) model <- "logistic"
 }# }}}
-if (class(cox)=="phreg")
+if (class(cox)[1]=="phreg")
 {# {{{
    p <- length(cox$coef)
    if (is.null(Z)) {
@@ -1174,10 +1195,10 @@ if (class(cox)=="phreg")
    if (cox$nstrata>1) {
 	   Z[,stratname] <- stratid
    }
-   model <- "fg"
+   model <-c(class(cox),is.null(cox$propodds))
    ## if (cox$prop.odds==TRUE) cox$model <- "logistic"
 }# }}}
-if (class(cox)=="crr")
+if (class(cox)[1]=="crr")
 {# {{{
    p <- length(cox$coef)
    if (is.null(Z)) stop("must give covariates for crr simulations\n");
@@ -1193,7 +1214,7 @@ if (class(cox)=="crr")
    Z <- Z[xid,,drop=FALSE]
    model <- "fg"
 }# }}}
-if (class(cox)=="comprisk")
+if (class(cox)[1]=="comprisk")
 {# {{{
    p <- length(cox$gamma)
    formula <- attr(cox, "Formula")
@@ -1219,10 +1240,9 @@ if (class(cox)=="comprisk")
 }# }}}
 
 out <- list(Z=Z,cum=cumhazard,rr=rr,id=xid,model=model)
-if (class(cox)=="phreg")
+if (class(cox)[1]=="phreg")
 	if (cox$nstrata>1) out <- c(out,list(strataid=stratid,strataname=stratname))
 return(out)
 
 }# }}}
 
-
